"""
MicroPython TinyRTC I2C Module, DS1307 RTC + AT24C32N EEPROM
https://github.com/mcauser/micropython-tinyrtc
"""
# AT24C32A, 32K (32768 kbit / 4 KB), 128 pages, 32 bytes per page, i2c addr 0x57

import time

AT24C32_ADR = 0x57                        # EEPROM-address

class AT24C32N(object):
    """Driver for the AT24C32N 32K EEPROM."""

    def __init__(self, i2c, i2c_addr=AT24C32_ADR, pages=128, bpp=32):
        self.i2c = i2c
        self.i2c_addr = i2c_addr
        self.pages = pages
        self.bpp = bpp # bytes per page

    def capacity(self):
        """Storage capacity in bytes"""
        return self.pages * self.bpp

    """  Text  """
    def read(self, addr, nbytes):
        """Read one or more bytes from the EEPROM starting from a specific address"""
        return self.i2c.readfrom_mem(self.i2c_addr, addr, nbytes, addrsize=16)

    def read_text(self, addr, num):
        self.i2c.writeto(self.i2c_addr, bytearray([addr//256, addr]))
        return self.i2c.readfrom(self.i2c_addr, num)

    def read_str(self, addr, num):
        self.i2c.writeto(self.i2c_addr, bytearray([addr//256, addr]))
        test = self.i2c.readfrom(self.i2c_addr, num)
        # print('test:', test)
        try:
            test1 = test.decode()
            # print('decode ok')
        except:
        #   test1 = '2020-11-25 22:33:44 12345 67890     '
            test1 = '0000000000000000000000000000000'
            # print('decode nicht ok')
        # print('test1:', test1)
        return test1


    def write(self, addr, buf):
        """Write one or more bytes to the EEPROM starting from a specific address"""
        offset = addr % self.bpp
        partial = 0
        # partial page write
        if offset > 0:
            partial = self.bpp - offset
            self.i2c.writeto_mem(self.i2c_addr, addr, buf[0:partial], addrsize=16)
            time.sleep_ms(5)
            addr += partial
        # full page write
        for i in range(partial, len(buf), self.bpp):
            self.i2c.writeto_mem(self.i2c_addr, addr+i-partial, buf[i:i+self.bpp], addrsize=16)
            time.sleep_ms(5)

    """  Byte  """
    def read_byte(self, addr):
        self.i2c.writeto(self.i2c_addr, bytearray([addr//256, addr]))
        t = self.i2c.readfrom(self.i2c_addr, 1)
        return t[0]
    def write_byte(self, addr, dat):
        self.i2c.writeto(self.i2c_addr, bytearray([addr//256, addr, dat]))

    """  Word  """
    def read_word(self, addr):
        self.i2c.writeto(self.i2c_addr, bytearray([addr//256, addr]))
        t = self.i2c.readfrom(self.i2c_addr, 2)
        return t[0]*256 + t[1]

    """  dWord  """
    def read_dword(self, addr):
        self.i2c.writeto(self.i2c_addr, bytearray([addr//256, addr]))
        t = self.i2c.readfrom(self.i2c_addr, 4)
        return (t[0]<<24) + (t[1]<<16) + (t[2]<<8) + t[3]

    """  Text   """
    def read_buf(self, addr, num):
        self.i2c.writeto(self.i2c_addr, bytearray([addr//256, addr]))
        return self.i2c.readfrom(self.i2c_addr, num)

#
